@@ -24,10 +24,8 @@ TODOs:
  * Handle RPV SUSY in ISAWIG
 """
 
-from __future__ import with_statement
-
 __author__ = "Andy Buckley <andy.buckley@cern.ch"
-__version__ = "0.4.0"
+__version__ = "0.4.1"
 
 
 def _autotype(var):
@@ -160,8 +158,10 @@ def readSLHAFile(spcfilename, **kwargs):
 
     Other keyword parameters are passed to readSLHA.
     """
-    with open(spcfilename, "r") as f:
-        return readSLHA(f.read(), kwargs)
+    f = open(spcfilename, "r")
+    rtn = readSLHA(f.read(), kwargs)
+    f.close()
+    return rtn
 
 
 def readSLHA(spcstr, ignorenobr=False):
@@ -248,8 +248,10 @@ def readISAWIGFile(isafilename, **kwargs):
 
     Other keyword parameters are passed to readSLHA.
     """
-    with open(isafilename, "r") as f:
-        return readISAWIG(f.read(), kwargs)
+    f = open(isafilename, "r")
+    rtn = readISAWIG(f.read(), kwargs)
+    f.close()
+    return rtn
 
 
 def readISAWIG(isastr, ignorenobr=False):
@@ -452,11 +454,13 @@ def writeSLHAFile(spcfilename, blocks, decays, **kwargs):
 
     Other keyword parameters are passed to writeSLHA.
     """
-    with open(spcfilename, "w") as f:
-        f.write(writeSLHA(blocks, decays, kwargs))
+    f = open(spcfilename, "w")
+    f.write(writeSLHA(blocks, decays, kwargs))
+    f.close()
 
 
-## TODO: Split writeSLHA into writeSLHA{Blocks,Decays}
+# TODO: Split writeSLHA into writeSLHA{Blocks,Decays}
+
 
 def writeSLHA(blocks, decays, ignorenobr=False):
     """
@@ -500,8 +504,9 @@ def writeISAWIGFile(isafilename, blocks, decays, **kwargs):
 
     TODO: Handle RPV SUSY
     """
-    with open(isafilename, "w") as f:
-        f.write(writeISAWIG(blocks, decays, kwargs))
+    f = open(isafilename, "w")
+    f.write(writeISAWIG(blocks, decays, kwargs))
+    f.close()
 
 
 def writeISAWIG(blocks, decays, ignorenobr=False):
@@ -634,28 +639,70 @@ def writeISAWIG(blocks, decays, ignorenobr=False):
             ## How to determine which to use?
             #decayout += str(i_d + 1) + " "
             decayout += "%d %e 0 " % (hwid, d.br)
-            ## TODO: Order decay products as required
-            ## Decaying Particle  	 Mode  	             Order of Decay Products
-            ## Top 	2 body mode: 	to Higgs 	     Higgs 	Bottom
-            ##  	3 body mode: 	via charged Higgs/W  quarks or leptons from W/Higgs 	Bottom
-            ## Gluino 	2 body modes: 	without gluon 	     Any Order
-            ## 		                with gluon 	     gluon 	colour neutral
-            ## 	        3-body modes: 	Rparity conserved    colour neutral 	q or qbar
-            ## Squark/Slepton 	2 body modes: Gaugino/Gluino with Quark/Lepton 	Gaugino quark
-            ##                                                                  Gluino 	lepton
-            ## 	                3 body modes: 	Weak 	     sparticle 	particles from W decay
-            ## Squark 	2 body modes: 	Lepton Number Violated 	quark 	lepton
-            ##           		baryon number violated 	quark 	quark
-            ## Slepton 	2 body modes: 	Lepton Number Violated 	q or qbar
-            ## Higgs 	2 body modes: 	(s)quark-(s)qbar 	(s)q or (s)qbar
-            ##          		(s)lepton-(s)lepton 	(s)l or (s)lbar
-            ## 	3 body modes: 		colour neutral 	q or qbar
-            ## 				l or lbar
-            ## Gaugino 	2 body modes: 	squark-quark 	q or sq
-            ## 		                slepton-lepton 	l or sl
-            ## 	3 body modes: 	R-parity conserved 	colour neutral 	q or qbar
-            ## 			                                   	l or lbar
-            ## Gaugino/Gluino 	3 body modes:              	R-parity violating 	Particles in the same order as the R-parity violating superpotential
+            ## Order decay products as required by HERWIG
+            ## Top
+            absids = map(abs, d.ids)
+            if abs(pid) == 6:
+                def cmp_bottomlast(a, b):
+                    """Comparison function which always puts b/bbar last"""
+                    if abs(a) == 5:
+                        return True
+                    elif abs(b) == 5:
+                        return False
+                    return cmp(a, b)
+                if len(absids) == 2:
+                    ## 2 body mode, to Higgs: Higgs; Bottom
+                    if (25 in absids or 26 in absids) and 5 in absids:
+                        d.ids = sorted(d.ids, key=cmp_bottomlast)
+                elif len(absids) == 3:
+                    ## 3 body mode, via charged Higgs/W: quarks or leptons from W/Higgs; Bottom
+                    if 37 in absids or 23 in absids:
+                        d.ids = sorted(d.ids, key=cmp_bottomlast)
+            ## Gluino
+            elif abs(pid) == 1000021:
+                if len(absids) == 2:
+                    ## 2 body mode
+                    ## without gluon: any order
+                    ## with gluon: gluon; colour neutral
+                    if 21 in absids:
+                        def cmp_gluonfirst(a, b):
+                            """Comparison function which always puts gluon first"""
+                            if a == 21:
+                                return False
+                            elif b == 21:
+                                return True
+                            return cmp(a, b)
+                        d.ids = sorted(d.ids, key=cmp_gluonfirst)
+                elif len(absids) == 3:
+                    ## 3-body modes, R-parity conserved: colour neutral; q or qbar
+                    pass
+
+            ## TODO: More sorting!
+            ## Squark/Slepton
+            ##   2 body modes: Gaugino/Gluino with Quark/Lepton     Gaugino      quark
+            ##                                                      Gluino       lepton
+            ##   3 body modes: Weak                                 sparticle     particles from W decay
+            ## Squark
+            ##   2 body modes: Lepton Number Violated               quark     lepton
+            ##                 Baryon number violated               quark     quark
+            ## Slepton
+            ##   2 body modes: Lepton Number Violated               q or qbar
+            ## Higgs
+            ##   2 body modes: (s)quark-(s)qbar                     (s)q or (s)qbar
+            ##                 (s)lepton-(s)lepton                  (s)l or (s)lbar
+            ##   3 body modes: Colour neutral                       q or qbar
+            ##                                                      l or lbar
+            ## Gaugino
+            ##   2 body modes: Squark-quark                         q or sq
+            ##                 Slepton-lepton                       l or sl
+            ##
+            ##   3 body modes: R-parity conserved   colour neutral  q or qbar
+            ##                                                      l or lbar
+
+            ## TODO: Gaugino/Gluino
+            ##   3 body modes:  R-parity violating:   Particles in the same order as the R-parity violating superpotential
+
+            ## Pad out IDs list with zeros
             ids = [0,0,0,0,0]
             for i, pid in enumerate(d.ids):
                 ids[i] = pid
